import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import OASResponse

OBJ_DATA = OASResponse.from_json("openapi stuff")


def test_oas_response_from_valid_json():
    resp = OASResponse.from_json("openapi stuff")
    assert resp.data == "openapi stuff"


def test_oas_response_from_valid_dict():
    resp = OASResponse.from_dict({"data": "openapi stuff"})
    assert resp.data == "openapi stuff"


def test_oas_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        OASResponse.from_dict({})


def test_oas_response_invalid_type():
    with pytest.raises(TypeError):
        OASResponse(data=1)


def test_oas_response_to_json():
    assert OBJ_DATA.to_json() == json.dumps({"data": "openapi stuff"})


def test_oas_response_to_dict():
    assert OBJ_DATA.to_dict() == {"data": "openapi stuff"}



"""This module contains domain models regrading found issues."""

from enum import Enum
from typing import Any, Dict, List

from mythx_models.base import JSONSerializable


class Severity(str, Enum):
    """An Enum holding the possible severities an issue can have."""

    UNKNOWN = "Unknown"
    NONE = "None"
    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"


class SourceType(str, Enum):
    """An Enum holding the possible source type values."""

    RAW_BYTECODE = "raw-bytecode"
    ETHEREUM_ADDRESS = "ethereum-address"
    SOLIDITY_CONTRACT = "solidity-contract"
    SOLIDITY_FILE = "solidity-file"


class SourceFormat(str, Enum):
    """An Enum holding the possible source format values."""

    TEXT = "text"
    SOLC_AST_LEGACY_JSON = "solc-ast-legacy-json"
    SOLC_AST_COMPACT_JSON = "solc-ast-compact-json"
    EVM_BYZANTIUM_BYTECODE = "evm-byzantium-bytecode"
    EWASM_RAW = "ewasm-raw"


class SourceMapLocation:
    def __init__(self, offset=0, length=0, file_id=-1, jump_type="-"):
        self.o = int(offset)
        self.l = int(length)
        self.f = int(file_id)
        self.j = jump_type

    @property
    def offset(self):
        return self.o

    @offset.setter
    def offset(self, value):
        value = int(value)
        if value <= 0:
            raise ValueError("Expected positive offset but received {}".format(value))
        self.o = int(value)

    @property
    def length(self):
        return self.l

    @length.setter
    def length(self, value):
        value = int(value)
        if value <= 0:
            raise ValueError("Expected positive length but received {}".format(value))
        self.l = int(value)

    @property
    def file_id(self):
        return self.f

    @file_id.setter
    def file_id(self, value):
        value = int(value)
        if value < -1:
            raise ValueError(
                "Expected positive file ID or -1 but received {}".format(value)
            )
        self.f = int(value)

    @property
    def jump_type(self):
        return self.j

    @jump_type.setter
    def jump_type(self, value):
        if value not in ("i", "o", "-"):
            raise ValueError(
                "Invalid jump type {}, must be one of i, o, -".format(value)
            )
        self.j = value

    def to_component_string(self):
        return "{}:{}:{}:{}".format(self.o, self.l, self.f, self.j)

    def __repr__(self):
        return "<SourceMapComponent ({})>".format(self.to_component_string())

    def __eq__(self, other: "SourceMapLocation"):
        return all(
            (self.o == other.o, self.l == other.l, self.f == other.f, self.j == other.j)
        )


class SourceMap:
    def __init__(self, source_map: str):
        self.components = self.decompress(source_map)

    @staticmethod
    def sourcemap_reducer(accumulator, component):
        parts = component.split(":")
        full = []
        for i in range(4):
            part_exists = i < len(parts) and parts[i]
            part = parts[i] if part_exists else accumulator[i]
            full.append(part)
        return full

    def decompress(self, source_map):
        components = source_map.split(";")
        accumulator = (-1, -1, -2, "")
        result = []

        for val in components:
            curr = self.sourcemap_reducer(accumulator, val)
            accumulator = curr
            result.append(curr)

        return [SourceMapLocation(*c) for c in result]

    def compress(self):
        compressed = []
        accumulator = (-1, -1, -2, "")
        for val in self.components:
            compr = []
            for i, v in enumerate((val.offset, val.length, val.file_id, val.jump_type)):
                if accumulator[i] == v:
                    compr.append("")
                else:
                    compr.append(str(v))
            accumulator = (val.offset, val.length, val.file_id, val.jump_type)
            compressed.append(":".join(compr).rstrip(":"))
        return ";".join(compressed)

    def to_sourcemap(self):
        return self.compress()

    def __eq__(self, other: "SourceMap"):
        return self.components == other.components


class SourceLocation(JSONSerializable):
    """The domain model for a source location in a detected issue."""

    def __init__(
        self,
        source_map: str,
        source_type: SourceType,
        source_format: SourceFormat,
        source_list: List[str],
    ):
        self.source_map = SourceMap(source_map)
        self.source_type = source_type
        self.source_format = source_format
        self.source_list = source_list

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.
        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """

        return cls(
            source_map=d["sourceMap"],
            source_type=SourceType(d["sourceType"].lower()),
            source_format=SourceFormat(d["sourceFormat"].lower()),
            source_list=d["sourceList"],
        )

    def to_dict(self):
        """Serialize the response model to a Python dict.
        :return: A dict holding the request model data
        """

        return {
            "sourceMap": self.source_map.to_sourcemap(),
            "sourceType": self.source_type,
            "sourceFormat": self.source_format,
            "sourceList": self.source_list,
        }

    def __eq__(self, other: "SourceLocation"):
        return all(
            (
                self.source_map.to_sourcemap() == other.source_map.to_sourcemap(),
                self.source_type == other.source_type,
                self.source_format == other.source_format,
                self.source_list == other.source_list,
            )
        )


class DecodedLocation(JSONSerializable):
    """A source location decoded by the API to line and column numbers."""

    def __init__(self, start_line, start_column, end_line, end_column):
        self.start_line = start_line
        self.start_column = start_column
        self.end_line = end_line
        self.end_column = end_column

    @classmethod
    def from_dict(cls, l: list):
        """Create the response domain model from a dict.

        :param l: The list to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """

        return cls(
            start_line=l[0]["line"],
            start_column=l[0]["column"],
            end_line=l[1]["line"],
            end_column=l[1]["column"],
        )

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """

        return [
            {"line": self.start_line, "column": self.start_column},
            {"line": self.end_line, "column": self.end_column},
        ]

    def __eq__(self, other: "DecodedLocation"):
        return all(
            (
                self.start_line == other.start_line,
                self.start_column == other.start_column,
                self.end_line == other.end_line,
                self.end_column == other.end_column,
            )
        )


class Issue(JSONSerializable):
    """The API response domain model for a single issue object."""

    def __init__(
        self,
        swc_id: str,
        swc_title: str,
        description_short: str,
        description_long: str,
        severity: Severity,
        locations: List[SourceLocation],
        extra: Dict[str, Any],
        decoded_locations: List[DecodedLocation] = None,
    ):
        self.swc_id = swc_id
        self.swc_title = swc_title
        self.description_short = description_short
        self.description_long = description_long
        self.severity = severity
        self.locations = locations
        self.decoded_locations = decoded_locations or []
        self.extra_data = extra

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """

        locations = [
            SourceLocation(
                source_map=loc.get("sourceMap"),
                source_type=loc.get("sourceType"),
                source_format=loc.get("sourceFormat"),
                source_list=loc.get("sourceList"),
            )
            for loc in d["locations"]
        ]

        decoded_locations = (
            [DecodedLocation.from_dict(l) for l in d.get("decodedLocations") if l]
            if d.get("decodedLocations")
            else []
        )

        return cls(
            swc_id=d["swcID"],
            swc_title=d["swcTitle"],
            description_short=d["description"]["head"],
            description_long=d["description"]["tail"],
            severity=Severity(d["severity"]) if d["severity"] else Severity.NONE,
            locations=locations,
            decoded_locations=decoded_locations,
            extra=d["extra"],
        )

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        result = {
            "swcID": self.swc_id,
            "swcTitle": self.swc_title,
            "description": {
                "head": self.description_short,
                "tail": self.description_long,
            },
            "severity": self.severity,
            "locations": [loc.to_dict() for loc in self.locations],
            "extra": self.extra_data,
        }
        if self.decoded_locations:
            result.update(
                {"decodedLocations": [loc.to_dict() for loc in self.decoded_locations]}
            )

        return result

    def __eq__(self, other: "Issue"):
        return all(
            (
                self.swc_id == other.swc_id,
                self.swc_title == other.swc_title,
                self.description_short == other.description_short,
                self.description_long == other.description_long,
                self.severity == other.severity,
                self.locations == other.locations,
                self.decoded_locations == other.decoded_locations,
                self.extra_data == other.extra_data,
            )
        )



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import VersionResponse

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/version-response.json")
OBJ_DATA = VersionResponse.from_json(JSON_DATA)


def assert_version_response(resp: VersionResponse):
    assert resp.api_version == DICT_DATA["api"]
    assert resp.maru_version == DICT_DATA["maru"]
    assert resp.mythril_version == DICT_DATA["mythril"]
    assert resp.harvey_version == DICT_DATA["harvey"]
    assert resp.hashed_version == DICT_DATA["hash"]


def test_version_response_from_valid_json():
    resp = VersionResponse.from_json(JSON_DATA)
    assert_version_response(resp)


def test_version_response_from_valid_dict():
    resp = VersionResponse.from_dict(DICT_DATA)
    assert_version_response(resp)


def test_version_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        VersionResponse.from_dict({})


def test_version_response_from_invalid_json():
    with pytest.raises(ValidationError):
        VersionResponse.from_json("{}")


def test_version_response_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_version_response_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



import json
from pathlib import Path


def get_test_case(path):
    with open(str(Path(__file__).parent / path)) as f:
        json_data = f.read()
        dict_data = json.loads(json_data)
    return json_data, dict_data



import json
from typing import Dict

from mythx_models.response.base import BaseResponse
from mythx_models.util import resolve_schema


class VersionResponse(BaseResponse):
    """The API response domain model for an API version request."""

    with open(resolve_schema(__file__, "version.json")) as sf:
        schema = json.load(sf)

    def __init__(
        self,
        api_version: str,
        maru_version: str,
        mythril_version: str,
        harvey_version: str,
        hashed_version: str,
    ):
        self.api_version = api_version
        self.maru_version = maru_version
        self.mythril_version = mythril_version
        self.harvey_version = harvey_version
        self.hashed_version = hashed_version

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(
            api_version=d["api"],
            maru_version=d["maru"],
            mythril_version=d["mythril"],
            harvey_version=d["harvey"],
            hashed_version=d["hash"],
        )

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "api": self.api_version,
            "maru": self.maru_version,
            "mythril": self.mythril_version,
            "harvey": self.harvey_version,
            "hash": self.hashed_version,
        }
        self.validate(d)
        return d

    def __eq__(self, other: "VersionResponse"):
        return all(
            (
                self.api_version == other.api_version,
                self.maru_version == other.maru_version,
                self.mythril_version == other.mythril_version,
                self.harvey_version == other.harvey_version,
                self.hashed_version == other.hashed_version,
            )
        )



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import DetectedIssuesRequest

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/detected-issues-request.json")
OBJ_DATA = DetectedIssuesRequest.from_json(JSON_DATA)


def test_analysis_issues_request_from_valid_json():
    req = DetectedIssuesRequest.from_json(json.dumps(DICT_DATA))
    assert req.uuid == OBJ_DATA.uuid
    assert req.method == "GET"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}


def test_analysis_issues_request_from_invalid_json():
    with pytest.raises(ValidationError):
        DetectedIssuesRequest.from_json("{}")


def test_analysis_issues_request_from_valid_dict():
    req = DetectedIssuesRequest.from_dict(DICT_DATA)
    assert req.uuid == OBJ_DATA.uuid


def test_analysis_issues_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        DetectedIssuesRequest.from_dict({})


def test_analysis_issues_request_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_analysis_issues_request_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



import json
from copy import deepcopy

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import Analysis, AnalysisStatusResponse
from mythx_models.util import serialize_api_timestamp

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/analysis-status-response.json")
OBJ_DATA = AnalysisStatusResponse.from_json(JSON_DATA)


def assert_analysis_data(analysis: Analysis):
    assert DICT_DATA["apiVersion"] == analysis.api_version
    assert DICT_DATA["maruVersion"] == analysis.maru_version
    assert DICT_DATA["mythrilVersion"] == analysis.mythril_version
    assert DICT_DATA["harveyVersion"] == analysis.harvey_version
    assert DICT_DATA["queueTime"] == analysis.queue_time
    assert DICT_DATA["runTime"] == analysis.run_time
    assert DICT_DATA["status"] == analysis.status
    assert DICT_DATA["submittedAt"] == serialize_api_timestamp(analysis.submitted_at)
    assert DICT_DATA["submittedBy"] == analysis.submitted_by
    assert DICT_DATA["uuid"] == analysis.uuid


def test_analysis_list_from_valid_json():
    resp = AnalysisStatusResponse.from_json(JSON_DATA)
    assert_analysis_data(resp.analysis)


def test_analysis_list_from_empty_json():
    with pytest.raises(ValidationError):
        AnalysisStatusResponse.from_json("{}")


def test_analysis_list_from_valid_dict():
    resp = AnalysisStatusResponse.from_dict(DICT_DATA)
    assert_analysis_data(resp.analysis)


def test_analysis_list_from_empty_dict():
    with pytest.raises(ValidationError):
        AnalysisStatusResponse.from_dict({})


def test_analysis_list_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA


def test_analysis_list_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_analysis_from_valid_json():
    analysis = Analysis.from_json(json.dumps(DICT_DATA))
    assert_analysis_data(analysis)


def test_analysis_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_analysis_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA


def test_analysis_propagate_error_field():
    analysis = deepcopy(OBJ_DATA.analysis)
    # add optional error field
    analysis.error = "err"
    analysis_dict = analysis.to_dict()
    assert analysis_dict["error"] == "err"


def test_analysis_from_valid_dict():
    analysis = Analysis.from_dict(DICT_DATA)
    assert_analysis_data(analysis)


def test_repr():
    analysis_repr = repr(OBJ_DATA.analysis)
    assert OBJ_DATA.uuid in analysis_repr
    assert OBJ_DATA.status in analysis_repr


def test_unexpected_param():
    data = deepcopy(DICT_DATA)
    data["extra_key"] = "test"
    analysis = Analysis.from_dict(data)
    assert_analysis_data(analysis)



import json
from typing import Dict

from mythx_models.response.base import BaseResponse
from mythx_models.util import resolve_schema


class AuthRefreshResponse(BaseResponse):
    """The API response domain model for a successful authentication refresh."""

    with open(resolve_schema(__file__, "auth.json")) as sf:
        schema = json.load(sf)

    def __init__(self, access_token: str, refresh_token: str):
        self.access_token = access_token
        self.refresh_token = refresh_token

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(
            access_token=d["jwtTokens"]["access"],
            refresh_token=d["jwtTokens"]["refresh"],
        )

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"jwtTokens": {"access": self.access_token, "refresh": self.refresh_token}}
        self.validate(d)
        return d

    def __eq__(self, other: "AuthRefreshResponse"):
        return all(
            (
                self.access_token == other.access_token,
                self.refresh_token == other.refresh_token,
            )
        )



"""This module contains the AuthLogoutRequest domain model."""

import json
from typing import Dict

from mythx_models.request.base import BaseRequest
from mythx_models.util import resolve_schema


class GroupCreationRequest(BaseRequest):
    """Perform an API request creates a new analysis group."""

    with open(resolve_schema(__file__, "group-creation.json")) as sf:
        schema = json.load(sf)

    def __init__(self, group_name: str = ""):
        self.group_name = group_name

    @property
    def endpoint(self):
        """The API's logout endpoint.

        :return: A string denoting the group endpoint without the host prefix
        """
        return "v1/analysis-groups"

    @property
    def method(self):
        """The HTTP method to perform.

        :return: The uppercase HTTP method, e.g. "POST"
        """
        return "POST"

    @property
    def parameters(self):
        """Additional URL parameters

        :return: A dict (str -> str) instance mapping parameter name to parameter content
        """
        return {}

    @property
    def headers(self):
        """Additional request headers.

        :return: A dict (str -> str) instance mapping header name to header content
        """
        return {}

    @property
    def payload(self):
        """The request's payload data.

        :return: A Python dict to be serialized into JSON format and submitted to the endpoint.
        """
        return {"groupName": self.group_name}

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the request domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(group_name=d["groupName"])

    def to_dict(self):
        """Serialize the request model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"groupName": self.group_name}
        self.validate(d)
        return d

    def __eq__(self, other: "GroupCreationRequest"):
        return self.group_name == other.group_name



import json
from datetime import datetime

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AnalysisListRequest

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/analysis-list-request.json")
OBJ_DATA = AnalysisListRequest.from_json(JSON_DATA)


def assert_analysis_list_request(req):
    assert req.offset == "test"
    assert req.date_from == datetime(2019, 2, 7, 0, 40, 49, 58158)
    assert req.date_to == datetime(2019, 2, 7, 0, 40, 49, 58158)
    assert req.payload == {}
    assert req.method == "GET"
    assert req.parameters == DICT_DATA
    assert req.headers == {}


def test_analysis_list_request_from_valid_json():
    req = AnalysisListRequest.from_json(JSON_DATA)
    assert_analysis_list_request(req)


def test_analysis_list_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisListRequest.from_json("{}")


def test_analysis_list_request_from_valid_dict():
    req = AnalysisListRequest.from_dict(DICT_DATA)
    assert_analysis_list_request(req)


def test_analysis_list_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisListRequest.from_dict({})


def test_analysis_list_request_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_analysis_list_request_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AnalysisInputResponse

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/analysis-input-response.json")
OBJ_DATA = AnalysisInputResponse.from_json(JSON_DATA)


def assert_submission_request(req: AnalysisInputResponse):
    assert req.contract_name == DICT_DATA["contractName"]
    assert req.bytecode == DICT_DATA["bytecode"]
    assert req.main_source == DICT_DATA["mainSource"]
    assert req.source_map == DICT_DATA["sourceMap"]
    assert req.deployed_bytecode == DICT_DATA["deployedBytecode"]
    assert req.deployed_source_map == DICT_DATA["deployedSourceMap"]
    assert req.sources == DICT_DATA["sources"]
    assert req.solc_version == DICT_DATA["version"]
    assert req.analysis_mode == DICT_DATA["analysisMode"]
    assert req.method == "POST"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {"data": DICT_DATA}


def test_analysis_submission_request_from_valid_json():
    req = AnalysisInputResponse.from_json(JSON_DATA)
    assert_submission_request(req)


def test_analysis_submission_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisInputResponse.from_json("{}")


def test_analysis_submission_request_from_valid_dict():
    req = AnalysisInputResponse.from_dict(DICT_DATA)
    assert_submission_request(req)


def test_analysis_submission_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisInputResponse.from_dict({})


def test_analysis_submission_request_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_analysis_submission_request_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA


def test_analysis_submission_request_bytecode_only():
    req = AnalysisInputResponse(bytecode=DICT_DATA["bytecode"])
    assert req.bytecode == DICT_DATA["bytecode"]
    assert req.analysis_mode == "quick"  # default value
    assert req.to_dict() == {"bytecode": DICT_DATA["bytecode"], "analysisMode": "quick"}
    assert json.loads(req.to_json()) == {
        "bytecode": DICT_DATA["bytecode"],
        "analysisMode": "quick",
    }


def test_analysis_submission_request_source_only():
    req = AnalysisInputResponse(sources=DICT_DATA["sources"])
    assert req.sources == DICT_DATA["sources"]
    assert req.analysis_mode == "quick"  # default value
    assert req.to_dict() == {"sources": DICT_DATA["sources"], "analysisMode": "quick"}
    assert json.loads(req.to_json()) == {
        "sources": DICT_DATA["sources"],
        "analysisMode": "quick",
    }


def test_analysis_submission_request_invalid_mode():
    req = AnalysisInputResponse(bytecode=DICT_DATA["bytecode"], analysis_mode="invalid")
    with pytest.raises(ValidationError):
        req.to_dict()


def test_analysis_submission_request_missing_field():
    req = AnalysisInputResponse()
    with pytest.raises(ValidationError):
        req.to_dict()



import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AuthLogoutResponse

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/auth-logout-response.json")
OBJ_DATA = AuthLogoutResponse.from_json(JSON_DATA)


def test_auth_login_response_from_valid_json():
    resp = AuthLogoutResponse.from_json("{}")
    assert type(resp) == AuthLogoutResponse


def test_auth_login_response_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLogoutResponse.from_json('{"foo": "bar"}')


def test_auth_login_response_from_valid_dict():
    resp = AuthLogoutResponse.from_dict({})
    assert type(resp) == AuthLogoutResponse


def test_auth_login_response_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLogoutResponse.from_dict({"foo": "bar"})


def test_auth_login_response_to_json():
    assert OBJ_DATA.to_json() == "{}"


def test_auth_login_response_to_dict():
    assert OBJ_DATA.to_dict() == {}



"""This module contains the base response domain model."""

import abc
import logging

from mythx_models.base import BaseModel

LOGGER = logging.getLogger(__name__)


class BaseResponse(BaseModel, abc.ABC):
    """An abstract object describing responses from the MythX API."""

    pass



import json
from pathlib import Path

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import GroupStatusRequest

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/group-status-request.json")
OBJ_DATA = GroupStatusRequest.from_json(JSON_DATA)


def assert_request(req: GroupStatusRequest):
    assert req.group_id == "test"
    assert req.method == "GET"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}
    assert req.endpoint == "v1/analysis-groups/test"


def test_from_invalid_json():
    with pytest.raises(ValidationError):
        GroupStatusRequest.from_json("{}")


def test_from_valid_dict():
    req = GroupStatusRequest.from_dict(json.loads(JSON_DATA))
    assert_request(req)


def test_from_invalid_dict():
    with pytest.raises(ValidationError):
        GroupStatusRequest.from_dict({})


def test_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



import json
from typing import Dict

from mythx_models.response.base import BaseResponse
from mythx_models.util import resolve_schema


class OASResponse(BaseResponse):
    """The API response domain model for an OpenAPI request."""

    with open(resolve_schema(__file__, "openapi.json")) as sf:
        schema = json.load(sf)

    def __init__(self, data: str):
        if not type(data) == str:
            raise TypeError("Expected data type str but got {}".format(type(data)))
        self.data = data

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(data=d["data"])

    @classmethod
    def from_json(cls, json_str: str):
        """

        :param json_str:
        :return:
        """
        # overwrite from base response because the API doesn't actually deliver
        # JSON but raw YAML/HTML
        return cls.from_dict({"data": json_str})

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"data": self.data}
        return d

    def __eq__(self, other: "OASResponse"):
        return self.data == other.data



import json
from copy import deepcopy
from datetime import datetime

import pytest
from dateutil.tz import tzutc

from mythx_models.exceptions import ValidationError
from mythx_models.response import (
    Group,
    GroupListResponse,
    GroupState,
    GroupStatistics,
    VulnerabilityStatistics,
)

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/group-list-response.json")
OBJ_DATA = GroupListResponse.from_json(JSON_DATA)


def assert_response_data(expected, g: Group):
    assert g.identifier == expected["id"]
    assert g.name == expected["name"]
    assert g.created_at == datetime(2019, 11, 4, 13, 38, 23, tzinfo=tzutc())
    assert g.created_by == "test"
    assert g.completed_at == datetime(2019, 11, 4, 13, 38, 23, tzinfo=tzutc())
    assert g.progress == 0
    assert g.status == GroupState.OPENED
    assert g.main_source_files == ["test"]
    assert g.analysis_statistics == GroupStatistics(
        total=0, queued=0, running=0, failed=0, finished=0
    )
    assert g.vulnerability_statistics == VulnerabilityStatistics(
        high=0, medium=0, low=0, none=0
    )


def test_from_valid_json():
    assert len(OBJ_DATA.groups) == 2
    for i, group in enumerate(OBJ_DATA.groups):
        assert_response_data(DICT_DATA["groups"][i], group)


def test_from_invalid_json():
    with pytest.raises(ValidationError):
        GroupListResponse.from_json("[]")


def test_from_empty_json():
    with pytest.raises(ValidationError):
        GroupListResponse.from_json("{}")


def test_from_valid_dict():
    resp = GroupListResponse.from_dict(DICT_DATA)
    assert len(resp.groups) == 2
    for i, group in enumerate(resp.groups):
        assert_response_data(DICT_DATA["groups"][i], group)


def test_from_invalid_dict():
    with pytest.raises(ValidationError):
        GroupListResponse.from_dict([])


def test_from_empty_dict():
    with pytest.raises(ValidationError):
        GroupListResponse.from_dict({})


def test_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA


def test_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_iteration():
    uuids = ("test1", "test2")
    # XXX: Don't use zip here to make sure __iter__ returns
    for idx, group in enumerate(OBJ_DATA):
        assert uuids[idx] == group.identifier


def test_valid_getitem():
    for idx, group in list(enumerate(OBJ_DATA.groups)):
        assert OBJ_DATA[idx] == group


def test_invalid_getitem_index():
    with pytest.raises(IndexError):
        OBJ_DATA[1337]


def test_oob_getitem_slice():
    OBJ_DATA[1337:9001] == []


def test_valid_delitem():
    group_list = deepcopy(OBJ_DATA)
    del group_list[0]
    assert group_list.groups == OBJ_DATA.groups[1:]
    assert group_list.total == OBJ_DATA.total - 1
    assert len(group_list) == OBJ_DATA.total - 1


def test_invalid_delitem():
    with pytest.raises(IndexError):
        del OBJ_DATA[1337]


def test_valid_setitem():
    group_list = deepcopy(OBJ_DATA)
    group_list[0] = "foo"
    assert group_list.groups[0] == "foo"
    assert group_list[0] == "foo"
    assert len(group_list.groups) == len(OBJ_DATA.groups)


def test_invalid_setitem():
    with pytest.raises(IndexError):
        OBJ_DATA[1337] = "foo"


def test_reversed():
    group_list = deepcopy(OBJ_DATA)
    assert list(reversed(group_list)) == list(reversed(OBJ_DATA.groups))
    assert group_list.total == OBJ_DATA.total


def test_valid_object_contains():
    assert OBJ_DATA.groups[0] in OBJ_DATA


def test_valid_uuid_contains():
    assert OBJ_DATA.groups[0].identifier in OBJ_DATA


def test_contains_error():
    with pytest.raises(ValueError):
        1 in OBJ_DATA


def test_invalid_object_contains():
    group = deepcopy(OBJ_DATA.groups[0])
    group.identifier = "something else"
    assert group not in OBJ_DATA


def test_invalid_uuid_contains():
    assert "foo" not in OBJ_DATA


def test_list_not_equals():
    group_list = deepcopy(OBJ_DATA)
    group_list.total = 0
    assert group_list != OBJ_DATA


def test_nested_not_equals():
    group_list = deepcopy(OBJ_DATA)
    group_list.groups[0].identifier = "foo"
    assert group_list != OBJ_DATA


def test_valid_equals():
    group_list = deepcopy(OBJ_DATA)
    assert group_list == OBJ_DATA



import json
from copy import deepcopy
from typing import Dict

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import (
    DetectedIssuesResponse,
    Issue,
    IssueReport,
    Severity,
    SourceFormat,
    SourceLocation,
    SourceType,
)

from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/detected-issues-response.json")
OBJ_DATA = DetectedIssuesResponse.from_json(JSON_DATA)


def assert_detected_issues(resp):
    assert len(resp.issue_reports) == 1
    report = resp.issue_reports[0]
    assert type(report) == IssueReport
    issue = report.issues[0]
    assert_issue(issue, DICT_DATA[0]["issues"][0])


def assert_issue(issue: Issue, data: Dict, skip_decoded: bool = False):
    assert issue.swc_id == data["swcID"]
    assert issue.swc_title == data["swcTitle"]
    assert issue.description_short == data["description"]["head"]
    assert issue.description_long == data["description"]["tail"]
    assert issue.severity == Severity(data["severity"])
    if not skip_decoded:
        assert issue.decoded_locations[0].to_dict() == data["decodedLocations"][0]
    assert len(issue.locations) == 1
    location = issue.locations[0]
    assert location.source_map.to_sourcemap() == data["locations"][0]["sourceMap"]
    assert location.source_format == SourceFormat(data["locations"][0]["sourceFormat"])
    assert location.source_type == SourceType(data["locations"][0]["sourceType"])
    assert location.source_list == data["locations"][0]["sourceList"]


def test_detected_issues_from_valid_json():
    resp = DetectedIssuesResponse.from_json(JSON_DATA)
    assert_detected_issues(resp)


def test_detected_issues_from_empty_json():
    resp = DetectedIssuesResponse.from_json("[]")
    assert resp.issue_reports == []


def test_detected_issues_from_dict():
    resp = DetectedIssuesResponse.from_dict(DICT_DATA)
    assert_detected_issues(resp)


def test_detected_issues_from_list():
    resp = DetectedIssuesResponse.from_dict([DICT_DATA[0]])
    assert_detected_issues(resp)


def test_detected_issues_from_invalid_type():
    with pytest.raises(ValidationError):
        DetectedIssuesResponse.from_dict("foo")


def test_detected_issues_from_empty_list():
    resp = DetectedIssuesResponse.from_dict([])
    assert resp.issue_reports == []


def test_detected_issues_from_invalid_dict():
    with pytest.raises(ValidationError):
        DetectedIssuesResponse.from_dict({})


def test_detected_issues_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_valid_swc_id_contains():
    assert DICT_DATA[0]["issues"][0]["swcID"] in OBJ_DATA


def test_valid_swc_id_not_contains():
    assert "SWC-104" not in OBJ_DATA


def test_invalid_key_contains():
    with pytest.raises(ValueError):
        1337 in OBJ_DATA


def test_response_length():
    resp = deepcopy(OBJ_DATA)
    total_report_issues = len(resp)
    assert len(resp) == total_report_issues
    resp.issue_reports.append(
        IssueReport(
            issues=["foo", "bar"],
            source_type=SourceType.RAW_BYTECODE,
            source_format=SourceFormat.EVM_BYZANTIUM_BYTECODE,
            source_list="foo.sol",
            meta_data={},
        )
    )
    assert len(resp) == total_report_issues + 2


def test_issue_iterator():
    for i, report in enumerate(OBJ_DATA.issue_reports):
        assert OBJ_DATA.issue_reports[i] == report


def test_report_iterator():
    for i, issue in enumerate(OBJ_DATA):
        assert issue == OBJ_DATA.issue_reports[0][i]


def test_issue_valid_getitem():
    assert OBJ_DATA.issue_reports[0] == OBJ_DATA[0]


def test_report_getitem():
    for i, issue in enumerate(OBJ_DATA):
        assert issue == OBJ_DATA.issue_reports[0][i]


def test_invalid_getitem():
    with pytest.raises(IndexError):
        test = OBJ_DATA[1337]


def test_valid_setitem():
    resp = deepcopy(OBJ_DATA)
    resp[0] = "foo"
    assert resp.issue_reports[0] == "foo"


def test_report_valid_setitem():
    resp = deepcopy(OBJ_DATA)
    resp.issue_reports[0][0] = "foo"
    assert resp.issue_reports[0][0] == "foo"


def test_invalid_setitem():
    with pytest.raises(TypeError):
        # string key on list access
        OBJ_DATA.issue_reports["foo"] = "bar"


def test_report_invalid_setitem():
    with pytest.raises(TypeError):
        # string key on list access
        OBJ_DATA.issue_reports[0]["foo"] = "bar"


def test_valid_delete():
    resp = deepcopy(OBJ_DATA)
    del resp[0]
    assert resp.issue_reports == []


def test_valid_report_delete():
    resp = deepcopy(OBJ_DATA)
    del resp[0][0]
    assert resp.issue_reports[0].issues == []


def test_invalid_delete():
    with pytest.raises(IndexError):
        del OBJ_DATA[1337]


def test_invalid_report_delete():
    with pytest.raises(IndexError):
        del OBJ_DATA[0][100]


def test_issue_from_valid_json():
    issue = Issue.from_json(json.dumps(DICT_DATA[0]["issues"][0]))
    assert_issue(issue, DICT_DATA[0]["issues"][0])


def test_issue_from_dict():
    issue = Issue.from_dict(DICT_DATA[0]["issues"][0])
    assert_issue(issue, DICT_DATA[0]["issues"][0])


def test_issue_to_json():
    assert (
        json.loads(OBJ_DATA.issue_reports[0][0].to_json()) == DICT_DATA[0]["issues"][0]
    )


def test_issue_to_dict():
    assert OBJ_DATA.issue_reports[0][0].to_dict() == DICT_DATA[0]["issues"][0]


def test_source_location_from_dict():
    loc2 = DICT_DATA[0]["issues"][0]["locations"][0]
    sl = SourceLocation.from_dict(loc2)
    assert sl.source_format == loc2["sourceFormat"]
    assert sl.source_list == loc2["sourceList"]
    assert sl.source_map.to_sourcemap() == loc2["sourceMap"]
    assert sl.source_type == loc2["sourceType"]


def test_decoded_locations_removed():
    issue_data = deepcopy(DICT_DATA[0]["issues"][0])
    del issue_data["decodedLocations"]
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, DICT_DATA[0]["issues"][0], skip_decoded=True)
    assert "decodedLocations" not in issue.to_dict()


def test_decoded_locations_empty_removed():
    issue_data = deepcopy(DICT_DATA[0]["issues"][0])
    issue_data["decodedLocations"] = []
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, DICT_DATA[0]["issues"][0], skip_decoded=True)
    assert "decodedLocations" not in issue.to_dict()


def test_decoded_locations_only_removed():
    issue_data = deepcopy(DICT_DATA[0]["issues"][0])
    issue_data["decodedLocations"] = [[]]
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, DICT_DATA[0]["issues"][0], skip_decoded=True)
    assert "decodedLocations" not in issue.to_dict()


def test_decoded_locations_empty_skip():
    issue_data = deepcopy(DICT_DATA[0]["issues"][0])
    issue_data["decodedLocations"].append([])
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, DICT_DATA[0]["issues"][0], skip_decoded=True)
    assert "decodedLocations" in issue.to_dict()


def test_decoded_locations_none_skip():
    issue_data = deepcopy(DICT_DATA[0]["issues"][0])
    issue_data["decodedLocations"] = None
    issue = Issue.from_dict(issue_data)
    assert_issue(issue, DICT_DATA[0]["issues"][0], skip_decoded=True)
    print(issue.to_dict())
    assert "decodedLocations" not in issue.to_dict()
